@@ -21,7 +21,8 @@
 #include "create_state_model.h"
 #include "r_interface/list_io.hpp"
 #include <Models/StateSpace/StateSpaceModelBase.hpp>
-#include <Models/StateSpace/MultivariateStateSpaceRegressionModel.hpp>
+#include <Models/StateSpace/Multivariate/MultivariateStateSpaceRegressionModel.hpp>
+#include <Models/StateSpace/Multivariate/StateModels/SharedStateModel.hpp>
 
 //==============================================================================
 // The functions declared here throw exceptions.  Code that uses them should be
@@ -34,19 +35,22 @@
 
 namespace BOOM {
 
-  // Forward declarations.  
+  // Forward declarations.
 
   // Host model.
   class MultivariateStateSpaceModelBase;
-  
+  class ConditionallyIndependentMultivariateStateSpaceModelBase;
+  using CIMSSMB = ConditionallyIndependentMultivariateStateSpaceModelBase;
+
   // State models.  This list will grow over time as more models are added.
-  class SharedLocalLevelStateModel;
+  //   class ConditionallyIndependentSharedLocalLevelStateModel;
 
   namespace bsts {
 
     // A factory for creating state components that are shared across multiple
     // time series.
-    class SharedStateModelFactory : public StateModelFactoryBase {
+    class ConditionallyIndependentSharedStateModelFactory
+        : public StateModelFactoryBase {
      public:
 
       // Args:
@@ -54,19 +58,19 @@ namespace BOOM {
       //   io_manager: A pointer to the object manaaging the R list that will
       //     record (or has already recorded) the MCMC output.  If a nullptr is
       //     passed then states will be created without IoManager support.
-      explicit SharedStateModelFactory(int nseries,
-                                       RListIoManager *io_manager)
+      explicit ConditionallyIndependentSharedStateModelFactory(
+          int nseries, RListIoManager *io_manager)
           : StateModelFactoryBase(io_manager),
             nseries_(nseries)
       {}
 
       using SharedStateModelVector =
           StateSpaceUtils::StateModelVector<SharedStateModel>;
-      
+
       // Adds all the state components listed in
       // r_state_specification_list to the model.
       // Args:
-      //   model: The model to which the state will be added.  
+      //   model: The model to which the state will be added.
       //   state_models: The state model vector holding the shared state models.
       //     This is typically owned by 'model'.
       //   r_state_specification_list: An R list of state components to be added
@@ -74,7 +78,7 @@ namespace BOOM {
       //     specification argument in bsts.
       //   prefix: An optional prefix added to the name of each state component.
       void AddState(SharedStateModelVector &state_models,
-                    MultivariateStateSpaceModelBase *model,
+                    CIMSSMB *model,
                     SEXP r_shared_state_specification,
                     const std::string &prefix = "");
 
@@ -90,14 +94,14 @@ namespace BOOM {
       //   list_element_name: The name of the final state vector in the R list
       //     holding the MCMC output.
       void SaveFinalState(
-          MultivariateStateSpaceModelBase *model,
+          CIMSSMB *model,
           BOOM::Vector *final_state = nullptr,
           const std::string &list_element_name = "final.shared.state");
-      
+
      private:
       // The number of time series being modeled.
       int nseries_;
-      
+
       // A factory function that unpacks information from an R object created by
       // AddXXX (where XXX is the name of a type of state model), and use it to
       // build the appropriate BOOM StateModel.  The specific R function
@@ -112,7 +116,7 @@ namespace BOOM {
       // Returns:
       //   A BOOM smart pointer to the appropriately typed MultivariateStateModel.
       Ptr<SharedStateModel> CreateSharedStateModel(
-          MultivariateStateSpaceModelBase *model,
+          CIMSSMB *model,
           SEXP r_state_component,
           const std::string &prefix);
 
@@ -120,13 +124,12 @@ namespace BOOM {
       // Specific functions to create specific state models.
       Ptr<SharedStateModel> CreateSharedLocalLevel(
           SEXP r_state_component,
-          MultivariateStateSpaceModelBase *model, 
+          CIMSSMB *model,
           const std::string &prefix);
     };
-    
+
   }  // namespace bsts
-  
+
 }  // namespace BOOM
 
 #endif  // BOOM_R_INTERFACE_CREATE_SHARED_STATE_MODEL_HPP_
-
